/*
i2c.c - A user-space program to access SMBus devices
Copyright (c) 2000-2002  JH Tseng <JHTseng@via.com.tw>
    
Usually, i2c devices are controlled by a kernel driver. But it is also
possible to access all devices on an adapter from userspace, through
the /dev interface. You need to load module i2c-dev for this. For VIA ChipSet,
you may need to load i2c-viapro module <modprobe i2c-viapro>

Each registered i2c adapter gets a number, counting from 0. You can
examine /proc/bus/i2c to see what number corresponds to which adapter.
I2C device files are character device files with major device number 89
and a minor device number corresponding to the number assigned as 
explained above. They should be called "i2c-%d" (i2c-0, i2c-1, ..., 
i2c-10, ...). All 256 minor device numbers are reserved for i2c.    

Now, you have to decide which adapter you want to access. You should
inspect /proc/bus/i2c to decide this. Adapter numbers are assigned
somewhat dynamically, so you can not even assume /dev/i2c-0 is the
first adapter.

Full interface description
==========================

The following IOCTLs are defined and fully supported 
(see also i2c-dev.h and i2c.h):

ioctl(file,I2C_SLAVE,long addr)
  Change slave address. The address is passed in the 7 lower bits of the
  argument (except for 10 bit addresses, passed in the 10 lower bits in this
  case).

ioctl(file,I2C_TENBIT,long select)
  Selects ten bit addresses if select not equals 0, selects normal 7 bit
  addresses if select equals 0.

ioctl(file,I2C_FUNCS,unsigned long *funcs)
  Gets the adapter functionality and puts it in *funcs.

ioctl(file,I2C_RDWR,struct i2c_ioctl_rdwr_data *msgset)

  Do combined read/write transaction without stop in between.
  The argument is a pointer to a struct i2c_ioctl_rdwr_data {

      struct i2c_msg *msgs;  // ptr to array of simple messages 
      int nmsgs;             // number of messages to exchange 
  }

  The msgs[] themselves contain further pointers into data buffers.
  The function will write or read data to or from that buffers depending
  on whether the I2C_M_RD flag is set in a particular message or not.
  The slave address and whether to use ten bit address mode has to be
  set in each message, overriding the values set with the above ioctl's.


Other values are NOT supported at this moment, except for I2C_SMBUS,
which you should never directly call; instead, use the access functions
below.

You can do plain i2c transactions by using read(2) and write(2) calls.
You do not need to pass the address byte; instead, set it through
ioctl I2C_SLAVE before you try to access the device.

You can do SMBus level transactions (see documentation file smbus-protocol 
for details) through the following functions:
  __s32 i2c_smbus_write_quick(int file, __u8 value);
  __s32 i2c_smbus_read_byte(int file);
  __s32 i2c_smbus_write_byte(int file, __u8 value);
  __s32 i2c_smbus_read_byte_data(int file, __u8 command);
  __s32 i2c_smbus_write_byte_data(int file, __u8 command, __u8 value);
  __s32 i2c_smbus_read_word_data(int file, __u8 command);
  __s32 i2c_smbus_write_word_data(int file, __u8 command, __u16 value);
  __s32 i2c_smbus_process_call(int file, __u8 command, __u16 value);
  __s32 i2c_smbus_read_block_data(int file, __u8 command, __u8 *values);
  __s32 i2c_smbus_write_block_data(int file, __u8 command, __u8 length, 
                                   __u8 *values);
All these tranactions return -1 on failure; you can read errno to see
what happened. The 'write' transactions return 0 on success; the
'read' transactions return the read value, except for read_block, which
returns the number of values read. The block buffers need not be longer
than 32 bytes.

The above functions are all macros, that resolve to calls to the
i2c_smbus_access function, that on its turn calls a specific ioctl
with the data in a specific format. Read the source code if you
want to know what happens behind the screens.

Related Link
============
1.Please refer to the linux/Documentation/i2c/dev-interface
2.Please refer to the lm_sensors web site for the i2c/smbus bus driver 
*/

#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <linux/i2c-dev.h>

int main(int argc, char *argv[])
{

   int file,i;
   int adapter_nr=0;	//Adapter number, assume 0
   int slave_addr,slave_cmd,count;
   char filename[20];
   int res;  

   if(argc < 5)
   {
      printf("Usage: smbus_read_byte_data <Adapter_num> <SMB_slave_addr> <SMB_slave_cmd> <Count>\n");
      exit(1);
   }
   
   adapter_nr=strtol(argv[1],0,16);
   slave_addr=strtol(argv[2],0,16);
   slave_cmd=strtol(argv[3],0,16);
   count=strtol(argv[4],0,16);
   sprintf(filename,"/dev/i2c-%d",adapter_nr);
   
   printf("dev:%s slave_addr:0x%02X slave_cmd:0x%02X count:0x%02X\n",
      filename,
      slave_addr,
      slave_cmd,
      count);
      
   if ((file = open(filename,O_RDWR)) < 0)
   {
      /* ERROR HANDLING; you cna check errno to see what went wrong */
      printf("open adapter file fail\n");
      exit(1);
   }
   
   if (ioctl(file,I2C_SLAVE,slave_addr) < 0) /* 7 bit address */
   {
      printf("ioctl set I2C_SLAVE address fail\n");
      exit(1);
   }
   
   for (i=0;i<count;i++)
   {
      res=i2c_smbus_read_byte_data(file,i);
   
      if (res < 0) 
      {
         /* ERROR HANDLING: i2c transaction failed */
         printf("i2c transaction fail\n");
         exit(0);
      }
      else 
      {
         /* res contains the read word */
         if (i%16==0)
            printf("\n");
         printf("%02X ",res);
      }     
   }
   printf("\n");

}
